require.paths.push('../lib');

var express = require('express')
  , proxy = require('proxy')
  , http_STATUS_CODES = require('http').STATUS_CODES
  , mime = require('mime')
  , fs = require('fs')
  , url = require('url')
  , path = require('path');

var httpProxy = module.exports =  proxy.createProxy();

httpProxy.middleware = proxy.middleware;
httpProxy.configure = function(opts) {
  this.save2Disk = opts.save2Disk || false;
  this.outputDir = opts.outputDir || '.';

  if (this.save2Disk) ensure_dir(this.outputDir);
}


//Ensure the dir exists.
ensure_dir = function(dir) {
  path.exists(dir, function(exists) {
    if (!exists) fs.mkdirSync(dir, 0777);
  });
};

//Merge hashes keys
merge = function(base, other) {
  Object.keys(other).forEach(function (k) {
    base[k] = other[k];
  });
}

//Generate random Id
_randomId = function() {
  var s = ''; 
  for (var i = 0; i < 4; i++) 
    s += Math.ceil(Math.pow(2,32) * Math.random()).toString(16);
  return s
}

httpProxy.on('start', function(data, state) {

  var req = data.req;

  var parsedUrl = url.parse(req.url);
  var pathName = parsedUrl.pathname || 'index.html';
  var mimeType = mime.lookup(pathName=='/'? 'index.html' : pathName);
  var d = new Date();
  merge(state, {
      id: _randomId()
    , startTime: d.getTime()
    , date: d.getFullYear() + '-' + (d.getMonth()+1) + '-' + d.getDate()
    , method: req.method
    , url: req.url
    , mimeType : mimeType
    , mimeTypeExtension : mime.extensions[mimeType]
    , request: {
        headers: req.headers
      , remoteAddress: req.socket.remoteAddress
      , remotePort: req.socket.remotePort
    }
    , response: {
        remoteAddress: req.connection.remoteAddress
      , remotePort: parsedUrl.port || 80
    }
  });

  //Save the request to file
  if (this.save2Disk) {
    var dirName = this.outputDir+state.date;
    ensure_dir(dirName);
    data.req.pipe(fs.createWriteStream(dirName+'/'+state.id+'.request.'+state.mimeTypeExtension));
  }
});

httpProxy.on('response', function(data, state) {

  //Save the response to file
  if (this.save2Disk) {
    var dirName = this.outputDir+state.date;
    ensure_dir(dirName);
    data.proxyRes.pipe(fs.createWriteStream(dirName+'/'+state.id+'.response.'+state.mimeTypeExtension));
  }

  data.proxyRes.once('end', function() {
    state.took = (new Date().getTime() - state.startTime) / 1000;
    state.statusCode = data.proxyRes.statusCode;
    state.statusCodeDesc = http_STATUS_CODES[data.proxyRes.statusCode];
    state.response.headers = data.proxyRes.headers;
    console.log(state.statusCode, data.req.url, state.id);
  });

})

